using JsonApiDotNetCore.Resources;

namespace JsonApiDotNetCore.Services
{
    /// <summary>
    /// Represents the foundational Resource Service layer in the JsonApiDotNetCore architecture that uses a Resource Repository for data access.
    /// </summary>
    /// <typeparam name="TResource">The resource type.</typeparam>
    public interface IResourceService<TResource> 
        : IResourceCommandService<TResource>, IResourceQueryService<TResource>, IResourceService<TResource, int> 
        where TResource : class, IIdentifiable<int>
    { }

    /// <summary>
    /// Represents the foundational Resource Service layer in the JsonApiDotNetCore architecture that uses a Resource Repository for data access.
    /// </summary>
    /// <typeparam name="TResource">The resource type.</typeparam>
    /// <typeparam name="TId">The resource identifier type.</typeparam>
    public interface IResourceService<TResource, in TId> 
        : IResourceCommandService<TResource, TId>, IResourceQueryService<TResource, TId>
        where TResource : class, IIdentifiable<TId>
    { }
}
